
{*******************************************************}
{                                                       }
{       4th GUI Library for Delphi                      }
{       FourthShortcutBar Unit                          }
{                                                       }
{       Copyright (C) 1996,2001 Sergey S. Tkachenko     }
{       e-mail: tkachenko@360.com.ua                    }
{          Web: http://www.4thfebruary.f2s.com          }
{                                                       }
{       Based on the TjanOutPanel by Jan Verhoeven      }
{                                                       }
{*******************************************************}

unit FthSBar;

{$I FTHVER.INC}

interface

uses
  Messages, Windows, SysUtils, Classes, Controls, Forms,
{$IFDEF DELPHI4_UP}
  ActnList, ImgList,
{$ENDIF}
  StdCtrls, ExtCtrls, Graphics;

type

{ TFourthShortcutBar }

  TFourthShortcutBarButtonState = (sbbsRaised, sbbsFlat, sbbsSunken);

  TFourthShortcutBar = class;
  TFourthShortcutBarPanel = class;
  TFourthShortcutBarButtons = class;

{ TFourthShortcutBarButton }

  TFourthShortcutBarButton = class(TCollectionItem)
  private
//    FAction: TCustomAction;
    FDisplayRect: TRect;
    FButtonRect: TRect;
    FText: string;
    FImageIndex: Integer;
    FOnClick: TNotifyEvent;
//    FVisible: Integer;
    procedure SetText(const Value: string);
    procedure SetImageIndex(const Value: Integer);
//    procedure SetVisible(Value: Boolean);
    procedure SetOnClick(const Value: TNotifyEvent);
  protected
{$IFDEF DELPHI3_UP}
    function GetDisplayName: string; override;
{$ENDIF}
    procedure Click;
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
  published
//    property Action: TCustomAction read FAction write FAction;
    property Text: string read FText write SetText;
    property ImageIndex: Integer read FImageIndex write SetImageIndex;
//    property Visible: Boolean read FVisible write SetVisible;
    property OnClick: TNotifyEvent read FOnClick write SetOnClick;
  end;

{ TFourthShortcutBarButtons }

  TFourthShortcutBarButtons = class(TCollection)
  private
    FFourthShortcutBarPanel: TFourthShortcutBarPanel;
    function GetItem(Index: Integer): TFourthShortcutBarButton;
    procedure SetItem(Index: Integer; Value: TFourthShortcutBarButton);
//    function GetVisibleCount: Integer;
  protected
{$IFDEF DELPHI3_UP}
    function GetOwner: TPersistent; override;
{$ENDIF}
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(FourthShortcutBarPanel: TFourthShortcutBarPanel);
    function Add: TFourthShortcutBarButton;
    property Items[Index: Integer]: TFourthShortcutBarButton read GetItem write SetItem; default;
//    property VisibleCount: Integer read GetVisibleCount;
  end;

{ TFourthShortcutBarPanel }

  TFourthShortcutBarPanel = class(TCollectionItem)
  private
    FDisplayRect: TRect;
    FText: string;
    FButtons: TFourthShortcutBarButtons;
    FOnClick: TNotifyEvent;
    FSmallImages: Boolean;
    procedure SetText(const Value: string);
    procedure SetButtons(const Value: TFourthShortcutBarButtons);
    procedure SetOnClick(const Value: TNotifyEvent);
    procedure SetSmallImages(const Value: boolean);
  protected
{$IFDEF DELPHI3_UP}
    function GetDisplayName: string; override;
{$ENDIF}
    procedure Click;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Text: string read FText write SetText;
    property Buttons: TFourthShortcutBarButtons read FButtons write SetButtons;
    property OnClick: TNotifyEvent read FOnClick write SetOnClick;
    property SmallImages: Boolean read FSmallImages write SetSmallImages;
  end;

{ TFourthShortcutBarPanels }

  TFourthShortcutBarPanels = class(TCollection)
  private
    FFourthShortcutBar: TFourthShortcutBar;
    function GetItem(Index: Integer): TFourthShortcutBarPanel;
    procedure SetItem(Index: Integer; Value: TFourthShortcutBarPanel);
  protected
{$IFDEF DELPHI3_UP}
    function GetOwner: TPersistent; override;
{$ENDIF}
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(FourthShortcutBar: TFourthShortcutBar);
    function Add: TFourthShortcutBarPanel;
    property Items[Index: Integer]: TFourthShortcutBarPanel read GetItem write SetItem; default;
  end;

{ TFourthShortcutBar }

  TDrawPanelEvent = procedure(FourthShortcutBar: TFourthShortcutBar;
    Panel: TFourthShortcutBarPanel; const Rect: TRect) of object;

  TFourthShortcutBar = class(TCustomControl)
  private
    FPanels: TFourthShortcutBarPanels;
    FPanelRect: TRect;        { rectangle for active panel buttons }
    FScrollTimer: TTimer;     { timer for scroll repeat }
    FScrollTop: Integer;      { first visible button }
    FScrollBottom: Integer;   { last visible button }
    FUpR: TRect;              { rect for up scroll button }
    FUpActive: Boolean;       { upscroller visible }
    FUpIsDown: Boolean;       { upscroller depressed }
    FDownR: TRect;            { rect for down scroll button }
    FDownActive: Boolean;     { downscroller visible }
    FDownIsDown: Boolean;     { downscroller depressed }
    FPanelIndex: Integer;     { current panel }
    FTabHeight: Integer;      { height of the clickable tabs }
    FPanelHeight: Integer;    { current height of the buttons area }
    FDownTab: Integer;        { current mousedown Tab }
    FDownButton: Integer;     { current mousedown Button }
    FMoveButton: Integer;     { current mousemove Button }
    FMoveTab: Integer;        { current mousemove Tab }
    FImages: TImagelist;      { large images, caption displayed below image }
    FSmallImages: TImageList; { small images, caption displayed right of image }
    procedure SetPanels(Value: TFourthShortcutBarPanels);
    procedure Update_Panel(Index: Integer; State: TFourthShortcutBarButtonState);
    procedure UpdatePanels(UpdateRects: Boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    function GetPanelAtPos(APoint: TPoint): Integer;
    procedure SetPanelIndex(const Value: Integer);
    procedure SetTabHeight(const Value: Integer);
    procedure SetImages(const Value: TImagelist);
    procedure UpdatePanel(First: Boolean; Index: Integer);
    procedure UpdateButton(aPanel, aButton: Integer;
      State: TFourthShortcutBarButtonstate);
    function GetButtonAtPos(Pt: TPoint): Integer;
    procedure SetSmallImages(const Value: TImageList);
    function GetButtonArea(pt: TPoint): Integer;
    procedure DrawDownRaised;
    procedure DrawDownSunken;
    procedure DrawUpRaised;
    procedure DrawUpSunken;
    procedure DrawRaised(R: TRect);
    procedure DrawUpArrow(R: TRect);
    procedure DrawDownArrow(R: TRect);
    procedure DrawSunken(R: TRect);
    procedure TimedScroll(Sender: TObject);
  protected
    procedure CMDesignHitTest(var Msg: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure ClickAtPos(APoint: TPoint);
    procedure CreateWnd; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
//    property Canvas: TCanvas read FCanvas;
  published
    property Align default alLeft;
    property Enabled;
    property Font stored True;
    property Images: TImageList read FImages write SetImages;
    property SmallImages: TImageList read FSmallImages write SetSmallImages;
    property Panels: TFourthShortcutBarPanels read FPanels write SetPanels;
    property PanelIndex: Integer read FPanelIndex write SetPanelIndex;
    property TabHeight: Integer read FTabHeight write SetTabHeight;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Hint;
    property Visible;
    property OnClick;
  {$IFDEF DELPHI5_UP}
    property OnContextPopup;
  {$ENDIF}
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  {$IFDEF DELPHI4_UP}
    property OnResize;
  {$ENDIF}
  end;

implementation

uses Consts;

{ TFourthShortcutBarPanel }

constructor TFourthShortcutBarPanel.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FButtons := TFourthShortcutBarButtons.Create(Self);
end;

procedure TFourthShortcutBarPanel.Assign(Source: TPersistent);
begin
  if Source is TFourthShortcutBarPanel then
  begin
    Text := TFourthShortcutBarPanel(Source).Text;
  end else
    inherited Assign(Source);
end;

{$IFDEF DELPHI3_UP}
function TFourthShortcutBarPanel.GetDisplayName: string;
begin
  Result := Text;
  if Result = '' then Result := inherited GetDisplayName;
end;
{$ENDIF}

procedure TFourthShortcutBarPanel.SetText(const Value: string);
begin
  if FText <> Value then
  begin
    FText := Value;
    Changed(False);
  end;
end;

procedure TFourthShortcutBarPanel.SetButtons(const Value: TFourthShortcutBarButtons);
begin
  FButtons.Assign(Value);
end;

destructor TFourthShortcutBarPanel.Destroy;
begin
  FButtons.Free;
  inherited;
end;

procedure TFourthShortcutBarPanel.SetOnClick(const Value: TNotifyEvent);
begin
  FOnClick := Value;
end;

procedure TFourthShortcutBarPanel.Click;
begin
  if Assigned(OnClick) then OnClick(Self);
end;

procedure TFourthShortcutBarPanel.SetSmallImages(const Value: boolean);
begin
  if (FSmallImages <> Value) then
  begin
    FSmallImages := Value;
    Changed(True);
  end;
end;

{ TFourthShortcutBarPanels }

constructor TFourthShortcutBarPanels.Create(FourthShortcutBar: TFourthShortcutBar);
begin
  inherited Create(TFourthShortcutBarPanel);
  FFourthShortcutBar := FourthShortcutBar;
end;

function TFourthShortcutBarPanels.Add: TFourthShortcutBarPanel;
begin
  Result := TFourthShortcutBarPanel(inherited Add);
end;

function TFourthShortcutBarPanels.GetItem(Index: Integer): TFourthShortcutBarPanel;
begin
  Result := TFourthShortcutBarPanel(inherited GetItem(Index));
end;

{$IFDEF DELPHI3_UP}
function TFourthShortcutBarPanels.GetOwner: TPersistent;
begin
  Result := FFourthShortcutBar;
end;
{$ENDIF}

procedure TFourthShortcutBarPanels.SetItem(Index: Integer; Value: TFourthShortcutBarPanel);
begin
  inherited SetItem(Index, Value);
end;

procedure TFourthShortcutBarPanels.Update(Item: TCollectionItem);
begin
  if Item <> nil then
    FFourthShortcutBar.Update_Panel(Item.Index, sbbsFlat) else
    FFourthShortcutBar.UpdatePanels(True);
end;

{ TFourthShortcutBar }

constructor TFourthShortcutBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csClickEvents, csDoubleClicks, csOpaque{$IFDEF DELPHI4_UP}, csActionClient{$ENDIF}];
  Canvas.Font := Font;
  Color := clBtnFace;
  Width := 100;
  Align := alLeft;
  FTabHeight := 22;
  FPanels := TFourthShortcutBarPanels.Create(Self);
  ParentFont := True;
  FMoveButton := -1;
  FMoveTab := -1;
end;

destructor TFourthShortcutBar.Destroy;
begin
  FPanels.Free;
  inherited Destroy;
end;

procedure TFourthShortcutBar.CreateWnd;
begin
  inherited CreateWnd;
  UpdatePanels(True);
end;

procedure TFourthShortcutBar.SetPanels(Value: TFourthShortcutBarPanels);
begin
  FPanels.Assign(Value);
end;

{   }

procedure TFourthShortcutBar.Update_Panel(Index: Integer;
  State: TFourthShortcutBarButtonState);
var
  S: string;
  R: TRect;
begin
  if HandleAllocated then
  begin
    with Panels[Index] do
    begin
      S := Text;
      R := FDisplayRect;
//      Canvas.Font := Self.Font;
      Canvas.Brush.Color := clBtnFace;
      Canvas.FillRect(R);
      Canvas.Brush.Style := bsClear;
      case State of
        sbbsSunken:
          begin
            Frame3D(Canvas, R, cl3dDkShadow, clBtnHighlight, 1);
            Frame3D(Canvas, R, clBtnShadow, cl3dLight, 1);
          end;
        sbbsFlat:
          begin
            if FPanelIndex = Index then
            begin
              Frame3D(Canvas, R, clBtnHighlight, cl3dDkShadow, 1);
              Inc(R.Right); Dec(R.Left); Dec(R.Top);
              Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
              Dec(R.Right); Inc(R.Left); Inc(R.Top);
            end else
            begin
              Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
              Inc(R.Right); Dec(R.Left); Dec(R.Top);
              Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
              Dec(R.Right); Inc(R.Left); Inc(R.Top);
            end;
          end;
        sbbsRaised:
          begin
            Frame3D(Canvas, R, clBtnHighlight, cl3dDkShadow, 1);
            Frame3D(Canvas, R, cl3dLight, clBtnShadow, 1);
          end;
      end;
      Dec(R.Right, 3); Inc(R.Left, 3);
      Canvas.Font.Color := clBtnText;
      DrawText(Canvas.Handle, @S[1], -1, R, DT_SINGLELINE or DT_CENTER or
        DT_VCENTER or DT_END_ELLIPSIS);
    end;
  end;
end;

procedure TFourthShortcutBar.UpdateButton(aPanel, aButton: Integer;
  State: TFourthShortcutBarButtonstate);
var
  R, btnR: Trect;
  btn: TFourthShortcutBarButton;
  img, h, w, dx, dy: Integer;
  s: string;
  small: Boolean;
begin
  if Panels.Count = 0 then Exit;
  if Panels[aPanel].Buttons.Count = 0 then Exit;
  btn := Panels[aPanel].Buttons[aButton];
  R := btn.FDisplayRect;
  btnR := btn.FButtonRect;
  small := Panels[aPanel].SmallImages;
  img := btn.ImageIndex;
  case State of
    sbbsSunken: Frame3D(Canvas, btnR, cl3DDkShadow,clBtnFace, 1);
    sbbsRaised: Frame3D(Canvas, btnR, clBtnFace, cl3DDkShadow, 1);
    sbbsFlat:   Frame3D(Canvas, btnR, clBtnShadow, clBtnShadow, 1);
  end;
  Canvas.Font.Color := clBtnHighlight;
  if Small then
  begin
    w := 16;
    h := 16;
    if Assigned(FSmallImages) then
    begin
      W := FSmallImages.Width;
      H := FSmallImages.height;
    end;
  end else
  begin
    W := 24;
    H := 24;
    if Assigned(FImages) then
    begin
      W := FImages.Width;
      H := FImages.height;
    end;
  end;

  dx := (btnR.Right - BtnR.Left - w) div 2;
  dy := (btnR.Bottom - BtnR.Top - h) div 2;

  if Small then
    FSmallImages.Draw(Canvas, btnR.left + dx, btnR.top + dy, img) else
    FImages.Draw(Canvas, btnR.left+dx, btnR.top + dy, img);

  S := btn.Text;

  if Small then
  begin
    R.Left := btnR.right + 4;
    DrawText(Canvas.Handle, @S[1], -1, R, DT_SINGLELINE or DT_LEFT or DT_END_ELLIPSIS);
  end else
  begin
    R.Top := btnR.Bottom + 2;
    DrawText(Canvas.Handle, @S[1], -1, R, DT_SINGLELINE or DT_CENTER or DT_TOP or DT_END_ELLIPSIS);
  end;
end;

// drawing of scroll buttons
procedure TFourthShortcutBar.DrawRaised(R: TRect);
begin
  with Canvas do
  begin
    Font := Self.Font;
    Brush.Color := clBtnFace;
    FillRect(R);
    Frame3D(Canvas, R, cl3dLight, cl3dDkShadow, 1);
    Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, 1);
    Brush.Style := bsClear;
  end;
end;

procedure TFourthShortcutBar.DrawSunken(R:TRect);
begin
  with Canvas do
  begin
    Font := Self.Font;
    Brush.Color := clBtnFace;
    FillRect(R);
    Pen.Color := cl3DDkShadow;
    MoveTo(R.Right-1,R.top);
    LineTo(R.left,R.top);
    lineto(R.left,R.bottom-1);
    pen.color:=cl3DLight;
    moveto(R.left,R.bottom);
    lineto(R.right,R.bottom);
    lineto(R.right,R.top);
    pen.color:=clBtnShadow;
    moveto(R.right-2,R.top+1);
    lineto(R.left+1,R.top+1);
    lineto(R.left+1,R.bottom-2);
    pen.color:=clBtnFace;
    moveto(R.left+1,R.bottom-1);
    lineto(R.right-1,R.bottom-1);
    lineto(R.right-1,R.top+1);
    brush.style:=bsclear;
  end;
end;

procedure TFourthShortcutBar.DrawUpArrow(R:TRect);
begin
  with canvas do
  begin
    Pen.Color := clBtnText;
    moveto(R.left+5,R.top+11);
    Lineto(R.left+13,R.top+11);
    moveto(R.left+6,R.top+10);
    Lineto(R.left+12,R.top+10);
    moveto(R.left+7,R.top+9);
    Lineto(R.left+11,R.top+9);
    moveto(R.left+8,R.top+8);
    Lineto(R.left+10,R.top+8);
    moveto(R.left+9,R.top+7);
    Lineto(R.left+9,R.top+7);
  end;
end;

procedure TFourthShortcutBar.DrawDownArrow(R:TRect);
begin
  with Canvas do
  begin
    Pen.Color := clBtnText;
    moveto(R.left+5, R.top+8);
    Lineto(R.left+13,R.top+8);
    moveto(R.left+6,R.top+9);
    Lineto(R.left+12,R.top+9);
    moveto(R.left+7,R.top+10);
    Lineto(R.left+11,R.top+10);
    moveto(R.left+8,R.top+11);
    Lineto(R.left+10,R.top+11);
    moveto(R.left+9,R.top+12);
    Lineto(R.left+9,R.top+12);
  end;
end;

procedure TFourthShortcutBar.DrawUpRaised;
begin
  DrawRaised(FUpR);
  DrawUpArrow(FUpR);
end;

procedure TFourthShortcutBar.DrawUpSunken;
begin
  DrawSunken(FUpR);
  DrawUpArrow(FUpR);
end;

procedure TFourthShortcutBar.DrawDownRaised;
begin
  DrawRaised(FDownR);
  DrawDownArrow(FDownR);
end;

procedure TFourthShortcutBar.DrawDownSunken;
begin
  DrawSunken(FDownR);
  DrawDownArrow(FDownR);
end;

procedure TFourthShortcutBar.UpdatePanel(First: Boolean; Index: Integer);
var
  I, Count, ButtonPos, W, H, dx: Integer;
  R, btnR: TRect;
  Small: Boolean;
  ZoneHeight: Integer;
  ZoneR: TRect;
begin
  R := FPanelRect;
  Canvas.Brush.Color := clBtnShadow;
  Canvas.FillRect(FPanelRect);

  FUpR := Rect(R.Right - 25, R.Top + 5, R.Right - 6, R.Top + 24);
  FUpActive := False;

  FDownR := Rect(R.Right - 25, R.Bottom - 25, R.Right - 6, R.Bottom - 6);
  FDownActive := False;

  Small := Panels[Index].SmallImages;
  if Panels[Index].Buttons = nil then Exit; // JH 30-jun-2001
  Count := Panels[Index].Buttons.Count;
//  Count := Panels[Index].Buttons.VisibleCount;
  if Count = 0 then Exit;
  ButtonPos := FPanelRect.Top + 2;
  if Small then
  begin
    if Assigned(FSmallImages) then
    begin
      H := FSmallImages.Height + 2;
      W := FSmallImages.Width + 2;
      ZoneHeight := H + 7;
      ZoneR := Rect(R.Left + 2, ButtonPos, R.Right - 2, ButtonPos + ZoneHeight - 1);
      btnR := Rect(ZoneR.Left + 2, ZoneR.Top + 2, ZoneR.Left + 4 + W, ZoneR.Top + 4 + H);
    end else
      Exit; // only display buttons when images assigned
  end else
{ if Large Images }
  begin
    if Assigned(FImages) then
    begin
      h := FImages.Height + 4;
      w := FImages.Width + 4;
      ZoneHeight := H + 22;
      ZoneR := Rect(R.Left + 2, ButtonPos, R.Right - 2, ButtonPos + ZoneHeight - 1);
      dx := (R.Right - R.Left - 4 - W) div 2;
      btnR := Rect(ZoneR.left + dx, ZoneR.Top + 4, ZoneR.Left + dx + 4 + W, ZoneR.Top + 6 + H);
    end else
     Exit; // only display buttons when images assigned
  end;
  if First then FScrollTop := 0;
  FScrollBottom := FScrollTop + ((FPanelRect.Bottom - FPanelRect.Top + 1) div ZoneHeight) - 1;
  if FScrollBottom>(count-1) then
   FScrollBottom:=count-1;
  // set button rects
  if Panels[Index].Buttons.Count > FScrollBottom then
//  if Panels[Index].Buttons.VisibleCount > FScrollBottom then
    for I := FScrollTop to FScrollBottom do
    begin
      Panels[Index].Buttons[I].FDisplayRect := ZoneR;
      Panels[Index].Buttons[I].FButtonRect := btnR;
    // next zone
      inc(ZoneR.top,ZoneHeight);
      inc(zoneR.bottom,Zoneheight);
      inc(btnR.top,ZoneHeight);
      inc(btnR.bottom,Zoneheight);
    end;
  // display buttons
  if Panels[index].Buttons.count>FScrollBottom then
    for i:=FScrollTop to FScrollBottom do
      UpdateButton(index,i,sbbsFlat);
    // check display of scroll buttons
//  timed:= assigned(FScrollTimer) and (FScrollTimer.enabled);
  if FScrollTop>0 then
  begin
    FUpActive:=true;
    if FUpIsDown then
      DrawUpSunken
    else
      DrawUpRaised;
  end
  else
    FUpActive:=false;
  if FScrollBottom<(count-1)then
  begin
    FDownActive:=true;
    if FDownIsDown then
      DrawDownSunken
    else
      DrawDownRaised;
  end
  else
   FDownActive:=false;
end;

procedure TFourthShortcutBar.UpdatePanels(UpdateRects: Boolean);
const
  MaxPanelCount = 128;
var
  I, Count, PanelPos,ot: Integer;
  R:TRect;
begin
  if HandleAllocated then
  begin
    R := ClientRect;
    Canvas.Font := Self.Font;
    Canvas.Brush.Color := clBtnShadow;
    Canvas.FillRect(R);
    Frame3D(Canvas, R, clBtnShadow, clBtnHighlight, 1);
    Frame3D(Canvas, R, cl3dDkShadow, clBtnShadow, 1);
    Count := Panels.Count;
    if UpdateRects then
    begin
      if Count > MaxPanelCount then Count := MaxPanelCount;
      if Count = 0 then
      begin
        //
      end else
      begin
        FPanelHeight := Height-2-(Count*FTabHeight);
        PanelPos := 1;
        for I := 0 to Count - 1 do
        begin
          Panels[I].FDisplayRect := Rect(1, PanelPos, Width-1, PanelPos+FTabHeight);
          Inc(PanelPos, FTabHeight);
          if I = PanelIndex then Inc(PanelPos, FPanelHeight);
        end;
      end;
    end;
    if count>0 then
    begin
    for I := 0 to Count - 1 do
      begin
        Update_Panel(I, sbbsFlat);
        if i = PanelIndex then
        begin
          ot:=Panels[i].FDisplayRect.bottom + 1;
          FPanelRect := Rect(2, ot, Width-3, ot + FPanelHeight - 2);
          UpdatePanel(True, i);
        end;
      end;
    end;
  end;
end;

procedure TFourthShortcutBar.CMFontChanged(var Message: TMessage);
begin
  inherited;
  UpdatePanels(True);
  Invalidate;  
end;

procedure TFourthShortcutBar.WMPaint(var Message: TWMPaint);
begin
  UpdatePanels(True);
  inherited;
end;

procedure TFourthShortcutBar.WMSize(var Message: TWMSize);
begin
  { Eat WM_SIZE message to prevent control from doing alignment }
{$IFDEF DELPHI4_UP}
  if not (csLoading in ComponentState) then Resize;
{$ENDIF}
  UpdatePanels(True);
end;

procedure TFourthShortcutBar.CMDesignHitTest(var Msg: TCMDesignHitTest);
const
  CTL_CLICK = 1;
begin
  inherited;
  if (Msg.Keys = CTL_CLICK) then ClickAtPos(Point(Msg.Pos.x, Msg.Pos.y));
end;

procedure TFourthShortcutBar.ClickAtPos(APoint: TPoint);
var
  Index: Integer;
begin
  Index := GetPanelAtPos(APoint);
  if (Index <> -1) then PanelIndex := Index;
end;

function TFourthShortcutBar.GetPanelAtPos(APoint: TPoint): Integer;
var
  I: Integer;
begin
  Result := -1;
  if Panels.Count = 0 then Exit;
  for I := 0 to Panels.Count - 1 do
  begin
    if PtInRect(Panels[I].FDisplayrect, APoint) then
    begin
      Result := I;
      Exit;
    end;
  end;
end;

procedure TFourthShortcutBar.SetPanelIndex(const Value: Integer);
begin
  FMoveButton := -1;
  FMoveTab := -1;
  if FPanelIndex <> Value then
  begin
    FPanelIndex := Value;
    UpdatePanels(True);
  end;
end;

procedure TFourthShortcutBar.SetTabHeight(const Value: Integer);
begin
  if FTabHeight <> Value then
  begin
   FTabHeight := Value;
   UpdatePanels(True);
  end;
end;

function TFourthShortcutBar.GetButtonArea(Pt: TPoint): Integer;
var
  I: Integer;
begin
  Result := -1;
  if PanelIndex = -1 then Exit;
  for I := FScrollTop to FScrollBottom do
  begin
    if PtInRect(Panels[PanelIndex].Buttons[I].FDisplayRect, Pt) then
    begin
      Result := I;
      Exit;
    end;
  end;
end;

function TFourthShortcutBar.GetButtonAtPos(pt:TPoint):integer;
var
  I: Integer;
begin
  Result := -1;
  if PanelIndex = -1 then exit;
  if Panels[PanelIndex].Buttons.count=0 then exit;
  for i:=FScrollTop to FScrollBottom do
  begin
    if ptInRect(Panels[PanelIndex].buttons[i].FButtonRect,pt) then
    begin
      result:=i;
      exit;
    end;
  end;
end;

procedure TFourthShortcutBar.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Index: Integer;
begin
  FDownTab := -1;
  FDownButton := -1;

  if FUpActive then // check if upscroll is pressed
  begin
    if PtInRect(FUpR, Point(X, Y)) then
    begin
      FUpIsDown := True;
      DrawUpSunken;
      FScrollTimer := TTimer.Create(Self);
      FScrollTimer.Interval := 200;
      FScrollTimer.OnTimer := TimedScroll;
      FScrollTimer.Enabled := True;
      Exit;
    end;
  end;

  if FDownActive then // check if downscroll is pressed
  begin
    if PtInRect(FDownR, Point(X, Y)) then
    begin
      FDownIsDown := True;
      DrawDownSunken;
      FScrollTimer := TTimer.Create(self);
      FScrollTimer.Interval := 200;
      FScrollTimer.OnTimer := TimedScroll;
      FScrollTimer.Enabled := True;
      Exit;
    end;
  end;

  Index := GetPanelAtPos(Point(X, Y));

  if (Index <> -1) then
  begin
    FDownTab := Index;
    FMoveButton := -1;
    Update_Panel(FDownTab, sbbsSunken);
  end else
  begin
    Index := GetButtonAtPos(Point(X, Y));
    if (Index <> -1) then
    begin
      FDownButton := Index;
      UpdateButton(PanelIndex, FDownButton, sbbsSunken);
    end;
  end;
end;

procedure TFourthShortcutBar.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Index: Integer;
begin
  if FUpIsDown then
  begin
    if Assigned(FScrollTimer) then FScrollTimer.Free;
    FUpIsDown := False;
//    DrawUpRaised;
    if FUpActive then
    begin
      Inc(FScrollTop, -1);
      UpdatePanel(False, PanelIndex);
      Exit;
    end;
  end;

  if FDownIsDown then
  begin
    if Assigned(FScrollTimer) then FScrollTimer.Free;
    FDownIsDown := False;
//    DrawDownRaised;
    if FDownActive then
    begin
      Inc(FScrollTop, 1);
      UpdatePanel(False, PanelIndex);
      Exit;
    end;
  end;

  if (FDownTab <> -1) then
  begin
    Update_Panel(FDownTab, sbbsFlat);
    Index := FDownTab;
    FDownTab := -1;
    PanelIndex := Index;
    Panels[PanelIndex].Click;
  end else
  if (FDownButton <> -1) then
  begin
    UpdateButton(PanelIndex, FDownButton, sbbsFlat);
    Index := FDownButton;
    FDownButton := -1;
    Panels[PanelIndex].Buttons[Index].Click;
  end;
end;

procedure TFourthShortcutBar.SetImages(const Value: TImagelist);
begin
  FImages := Value;
end;

procedure TFourthShortcutBar.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = FImages) then
    SetImages(nil);
  if (Operation = opRemove) and (AComponent = FSmallImages) then
    SetSmallImages(nil);
end;

procedure TFourthShortcutBar.SetSmallImages(const Value: TImageList);
begin
  FSmallImages := Value;
end;

procedure TFourthShortcutBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Index: Integer;
  I: Integer;
begin
  Index := GetPanelAtPos(Point(X, Y));

{$IFDEF DELPHI3_UP}
  if Index <> -1 then
    Cursor := crHandPoint else
{$ENDIF}
    Cursor := crDefault;

  if (not (ssLeft in Shift)) and (Index <> FMoveTab) and
    (Panels.Count <> 0) then
  begin
    for I := 0 to Pred(Panels.Count) do
    begin
      if I <> Index then
        Update_Panel(I, sbbsFlat) else
        Update_Panel(I, sbbsRaised);
    end;
    FMoveTab:=index;
  end;

  if not (ssLeft in Shift) then
  begin
    if (PanelIndex <> -1) and (Panels[PanelIndex].Buttons.Count > 0) then
    begin
      Index := GetButtonArea(Point(X, Y));
      if (Index <> FMoveButton) then
      begin
        for I := FScrollTop to FScrollBottom do
        if I <> Index then
          UpdateButton(PanelIndex, I, sbbsFlat) else
          UpdateButton(PanelIndex, Index, sbbsRaised);
        FMoveButton := Index;
      end;
    end;
  end; { if not (ssLeft in Shift) ... }
end;

procedure TFourthShortcutBar.TimedScroll(Sender: TObject);
begin
  if FUpIsDown then
  begin
    if FUpActive then
    begin
      Inc(FScrollTop, -1);
      UpdatePanel(False, PanelIndex);
    end;
    if not FUpActive then
      if Assigned(FScrollTimer) then
        FScrollTimer.Enabled := False;
  end else
  if FDownIsDown then
  begin
    if FDownActive then
    begin
      Inc(FScrollTop, 1);
      UpdatePanel(False, PanelIndex);
    end;
    if not FDownActive then
      if Assigned(FScrollTimer) then
        FScrollTimer.Enabled := False;
  end else
  if Assigned(FScrollTimer) then
    FScrollTimer.Enabled := False;
end;

{ TFourthShortcutBarButton }

procedure TFourthShortcutBarButton.Assign(Source: TPersistent);
begin
  if (Source is TFourthShortcutBarButton) then
  begin
    Text := TFourthShortcutBarButton(Source).Text;
    ImageIndex := TFourthShortcutBarButton(Source).ImageIndex;
    OnClick := TFourthShortcutBarButton(Source).OnClick;
  end else
    inherited Assign(Source);
end;

procedure TFourthShortcutBarButton.Click;
begin
  if Assigned(OnClick) then OnClick(self);
end;

constructor TFourthShortcutBarButton.Create(Collection: TCollection);
begin
  inherited Create(Collection);
end;

{$IFDEF DELPHI3_UP}
function TFourthShortcutBarButton.GetDisplayName: string;
begin
  Result := Text;
  if Result = '' then Result := inherited GetDisplayName;
end;
{$ENDIF}

procedure TFourthShortcutBarButton.SetImageIndex(const Value: integer);
begin
  if FImageIndex<>value then
  begin
    FImageIndex := Value;
    Changed(False)
  end;
end;

procedure TFourthShortcutBarButton.SetOnClick(const Value: TNotifyEvent);
begin
  FOnClick := Value;
end;

procedure TFourthShortcutBarButton.SetText(const Value: string);
begin
  if FText <> Value then
  begin
    FText := Value;
    Changed(False);
  end;
end;

{ TFourthShortcutBarButtons }

function TFourthShortcutBarButtons.Add: TFourthShortcutBarButton;
begin
  Result := TFourthShortcutBarButton(inherited Add);
end;

constructor TFourthShortcutBarButtons.Create(FourthShortcutBarPanel: TFourthShortcutBarPanel);
begin
  inherited Create(TFourthShortcutBarButton);
  FFourthShortcutBarPanel := FourthShortcutBarPanel;
end;

function TFourthShortcutBarButtons.GetItem(Index: Integer): TFourthShortcutBarButton;
begin
  Result := TFourthShortcutBarButton(inherited GetItem(Index));
end;

{$IFDEF DELPHI3_UP}
function TFourthShortcutBarButtons.GetOwner: TPersistent;
begin
  Result := FFourthShortcutBarPanel;
end;
{$ENDIF}

procedure TFourthShortcutBarButtons.SetItem(Index: Integer; Value: TFourthShortcutBarButton);
begin
  inherited SetItem(Index, Value);
end;

procedure TFourthShortcutBarButtons.Update(Item: TCollectionItem);
begin
  if (Item <> nil) then FFourthShortcutBarPanel.Changed(False);
end;

end.
